@@ -1,7 +1,7 @@
 # test for xml.dom.minidom
 
 import pickle
-from test.support import verbose, run_unittest, findfile
+from test.support import run_unittest, findfile
 import unittest
 
 import xml.dom.minidom
@@ -310,9 +310,10 @@ class MinidomTest(unittest.TestCase):
         self.confirm(len(child.attributes) == 0
                 and child.getAttributeNode("spam") is None)
         dom2 = Document()
-        child2 = dom2.appendChild(dom.createElement("foo"))
-        self.assertRaises(xml.dom.NotFoundErr, child.removeAttributeNode,
-            node)
+        child2 = dom2.appendChild(dom2.createElement("foo"))
+        node2 = child2.getAttributeNode("spam")
+        self.assertRaises(xml.dom.NotFoundErr, child2.removeAttributeNode,
+            node2)
         dom.unlink()
 
     def testHasAttribute(self):
@@ -607,7 +608,7 @@ class MinidomTest(unittest.TestCase):
     def testHasChildNodes(self):
         dom = parseString("<doc><foo/></doc>")
         doc = dom.documentElement
-        self.assertTrue(dom.hasChildNodes())
+        self.assertTrue(doc.hasChildNodes())
         dom2 = parseString("<doc/>")
         doc2 = dom2.documentElement
         self.assertFalse(doc2.hasChildNodes())
@@ -1073,7 +1074,7 @@ class MinidomTest(unittest.TestCase):
             '<?xml version="1.0" encoding="utf-16"?>'
             '<foo>\u20ac</foo>'.encode('utf-16'))
 
-        # Verify that character decoding errors throw exceptions instead
+        # Verify that character decoding errors raise exceptions instead
         # of crashing
         self.assertRaises(UnicodeDecodeError, parseString,
                 b'<fran\xe7ais>Comment \xe7a va ? Tr\xe8s bien ?</fran\xe7ais>')
@@ -1517,6 +1518,10 @@ class MinidomTest(unittest.TestCase):
         doc2 = parseString(doc.toxml())
         self.confirm(doc2.namespaceURI == xml.dom.EMPTY_NAMESPACE)
 
+    def testExceptionOnSpacesInXMLNSValue(self):
+        with self.assertRaisesRegex(ValueError, 'Unsupported syntax'):
+            parseString('<element xmlns:abc="http:abc.com/de f g/hi/j k"><abc:foo /></element>')
+
     def testDocRemoveChild(self):
         doc = parse(tstfile)
         title_tag = doc.documentElement.getElementsByTagName("TITLE")[0]
