@@ -32,13 +32,14 @@ except ImportError:
                          "no sophisticated Python source file search will be done.", file=sys.stderr)
 
 
-decl_re = re.compile(rb"coding[=:]\s*([-\w.]+)")
+decl_re = re.compile(rb'^[ \t\f]*#.*coding[:=][ \t]*([-\w.]+)')
+blank_re = re.compile(rb'^[ \t\f]*(?:[#\r\n]|$)')
 
 def get_declaration(line):
-    match = decl_re.search(line)
+    match = decl_re.match(line)
     if match:
         return match.group(1)
-    return ''
+    return b''
 
 def has_correct_encoding(text, codec):
     try:
@@ -58,7 +59,8 @@ def needs_declaration(fullpath):
         line1 = infile.readline()
         line2 = infile.readline()
 
-        if get_declaration(line1) or get_declaration(line2):
+        if (get_declaration(line1) or
+            blank_re.match(line1) and get_declaration(line2)):
             # the file does have an encoding declaration, so trust it
             return False
 
